using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;

namespace SpringClient
{
	/// <summary>
	/// Summary description for Chatroom.
	/// </summary>
	public class Chatroom : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.ListBox ChatMembers;
		private System.Windows.Forms.Splitter splitter1;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.RichTextBox MessageType;
		private System.Windows.Forms.Splitter splitter2;
		private System.Windows.Forms.RichTextBox Messages;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private SortedList UsersInside;

		private struct UserInfo
		{
			public string name;
			public string clan;
		}

		/*
		private struct ScrollInfo
		{
			public UInt32 cbSize;
			public UInt32 fMask;
			public Int32 nMin;
			public Int32 nMax;
			public UInt32 nPage;
			public Int32 nPos;
			public Int32 nTrackPos;
		}

		*/

		public Guid ID;

		[DllImport("User32.dll")]
		static extern Int32 SendMessage(IntPtr hWnd, Int32 wMsg, Int32 wParam, Int32 lParam);

		[DllImport("User32.dll")]
			static extern Int32 GetScrollPos(IntPtr hWnd, Int32 nBar);

		[DllImport("User32.dll")]
			static extern Boolean GetScrollRange(IntPtr hWnd, Int32 nBar, ref Int32 lpMinPos,
				ref Int32 lpMaxPos);

		/*
		[DllImport("User32.dll")]
		static extern Boolean GetScrollInfo(IntPtr hWnd, Int32 fnBar,
			[MarshalAs(UnmanagedType.LPStruct)] ScrollInfo lpsi);
*/
		public Chatroom()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitializeComponent call
			UsersInside = new SortedList();
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.ChatMembers = new System.Windows.Forms.ListBox();
			this.splitter1 = new System.Windows.Forms.Splitter();
			this.panel1 = new System.Windows.Forms.Panel();
			this.Messages = new System.Windows.Forms.RichTextBox();
			this.splitter2 = new System.Windows.Forms.Splitter();
			this.MessageType = new System.Windows.Forms.RichTextBox();
			this.panel1.SuspendLayout();
			this.SuspendLayout();
			// 
			// ChatMembers
			// 
			this.ChatMembers.Dock = System.Windows.Forms.DockStyle.Right;
			this.ChatMembers.IntegralHeight = false;
			this.ChatMembers.Location = new System.Drawing.Point(344, 0);
			this.ChatMembers.Name = "ChatMembers";
			this.ChatMembers.Size = new System.Drawing.Size(112, 456);
			this.ChatMembers.TabIndex = 0;
			// 
			// splitter1
			// 
			this.splitter1.Dock = System.Windows.Forms.DockStyle.Right;
			this.splitter1.Location = new System.Drawing.Point(341, 0);
			this.splitter1.Name = "splitter1";
			this.splitter1.Size = new System.Drawing.Size(3, 456);
			this.splitter1.TabIndex = 1;
			this.splitter1.TabStop = false;
			// 
			// panel1
			// 
			this.panel1.Controls.Add(this.Messages);
			this.panel1.Controls.Add(this.splitter2);
			this.panel1.Controls.Add(this.MessageType);
			this.panel1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.panel1.Location = new System.Drawing.Point(0, 0);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(341, 456);
			this.panel1.TabIndex = 2;
			// 
			// Messages
			// 
			this.Messages.Dock = System.Windows.Forms.DockStyle.Fill;
			this.Messages.Location = new System.Drawing.Point(0, 0);
			this.Messages.Name = "Messages";
			this.Messages.ReadOnly = true;
			this.Messages.ScrollBars = System.Windows.Forms.RichTextBoxScrollBars.Vertical;
			this.Messages.Size = new System.Drawing.Size(341, 405);
			this.Messages.TabIndex = 2;
			this.Messages.Text = "";
			this.Messages.TextChanged += new System.EventHandler(this.Messages_TextChanged);
			this.Messages.LinkClicked += new System.Windows.Forms.LinkClickedEventHandler(this.Messages_LinkClicked);
			// 
			// splitter2
			// 
			this.splitter2.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.splitter2.Location = new System.Drawing.Point(0, 405);
			this.splitter2.Name = "splitter2";
			this.splitter2.Size = new System.Drawing.Size(341, 3);
			this.splitter2.TabIndex = 1;
			this.splitter2.TabStop = false;
			// 
			// MessageType
			// 
			this.MessageType.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.MessageType.Location = new System.Drawing.Point(0, 408);
			this.MessageType.Name = "MessageType";
			this.MessageType.Size = new System.Drawing.Size(341, 48);
			this.MessageType.TabIndex = 0;
			this.MessageType.Text = "";
			this.MessageType.KeyDown += new System.Windows.Forms.KeyEventHandler(this.MessageType_KeyDown);
			// 
			// Chatroom
			// 
			this.Controls.Add(this.panel1);
			this.Controls.Add(this.splitter1);
			this.Controls.Add(this.ChatMembers);
			this.Name = "Chatroom";
			this.Size = new System.Drawing.Size(456, 456);
			this.panel1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void MessageType_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			if ( e.Modifiers != Keys.Shift && e.KeyCode == Keys.Enter )
			{
				MessageSend();
				e.Handled = true;
			}
		}

		public bool AddUser(int userid, string name, string clan, bool notify)
		{
			if ( UsersInside.Contains(userid) )
			{
				// dont add a user twice, just ignore this call
				return false;
			}
			if ( notify == true )
			{
				if ( clan == "" )
					this.Messages.AppendText(string.Format("{0} has logged in.\n", name));
				else
					this.Messages.AppendText(string.Format("{0} has logged in.\n", clan.Replace(":",name)));
				Messages.Update();
			}


			UserInfo ui;
			ui.name = (string)name.Clone();
			ui.clan = (string)clan.Clone();

			UsersInside.Add(userid, ui);

			if ( clan == "" )
				ChatMembers.Items.Add(name);
			else
				ChatMembers.Items.Add(clan.Replace(":",name));
			return true;
		}

		public bool RemoveUser(int userid)
		{
			if ( !UsersInside.Contains(userid) )
			{
				return false;
			}
			UserInfo ui = (UserInfo)UsersInside[userid];
			if ( ui.clan == "" )
				this.Messages.AppendText(string.Format("{0} has logged out.\n", ui.name));
			else
				this.Messages.AppendText(string.Format("{0} has logged out.\n", ui.clan.Replace(":",ui.name)));
			Messages.Update();

			UsersInside.Remove(userid);
			if ( ui.clan == "" )
				ChatMembers.Items.Remove(ui.name);
			else
				ChatMembers.Items.Remove(ui.clan.Replace(":",ui.name));
			return true;
		}

		public void PostMessage( int userid, string message )
		{
			if ( UsersInside.ContainsKey(userid)  )
			{
				UserInfo ui = (UserInfo)UsersInside[userid];
				Font backup = Messages.SelectionFont;
				Messages.SelectionFont = new Font(Messages.Font, FontStyle.Bold);//new Font("Microsoft Sans Serif", 8.5f, FontStyle.Bold);
				Messages.AppendText(ui.name.ToString() + ": ");
				Messages.SelectionFont = backup;
			}
			else
				Messages.AppendText(userid.ToString() + ": ");
			Messages.AppendText(message + "\n");
			Messages.Update();

			/*
			Int32 minPos = 0, maxPos = 0;
			GetScrollRange(Messages.Handle, ClientControls.Win32.SB_VERT, ref minPos, ref maxPos);
			int pos = GetScrollPos(Messages.Handle, ClientControls.Win32.SB_VERT);
			if ( pos == maxPos )
			*/
			/*
			ScrollInfo si = new ScrollInfo();
			si.fMask = 0x1 | 0x2 | 0x4 | 0x10;
			si.cbSize = 28;
			GetScrollInfo(Messages.Handle, ClientControls.Win32.SB_VERT, si);
			*/
			if ( !Messages.Focused )
				SendMessage(Messages.Handle, Win32.WM_VSCROLL, Win32.SB_BOTTOM, 0);
			
		}

		private void MessageSend()
		{
			if ( MessageType.Text.Length == 0 )
				return;

			if ( MessageType.Text.Length > CONSTANTS.MAXMESSAGELEN )
			{
				MessageBox.Show(string.Format("Message cannot be longer than {0} characters", CONSTANTS.MAXMESSAGELEN));
				MessageType.Clear();
				return;
			}

			byte[] msg = new byte[19];
			msg[0] = (byte)PACKETS.PACK_CHATMESSAGE;
			Phoenix.ByteConverter.ToNetworkBytes(
				(ushort)MessageType.Text.Length,msg,1);
			Phoenix.ByteConverter.ToNetworkBytes(ID,msg,3);

			
			Global.connMan.Send(msg);
			//Global.connMan.Send((ushort)MessageType.Text.Length);
			//Global.connMan.Send(ID);
			Global.connMan.Send(MessageType.Text);
			
			MessageType.Clear();
		}

		private void Messages_TextChanged(object sender, System.EventArgs e)
		{
		
		}

		private void Messages_LinkClicked(object sender, System.Windows.Forms.LinkClickedEventArgs e)
		{
			if ( DialogResult.Yes == MessageBox.Show("Are you sure you want to launch this link?", "Link clicked", MessageBoxButtons.YesNo) )
				System.Diagnostics.Process.Start(e.LinkText);
		}
	}
}
